<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:tt="http://xmlns.oracle.com/pcbpel/taskservice/tasktype"
xmlns:bpel="http://schemas.xmlsoap.org/ws/2003/03/business-process/">

<xsl:include href="/oracle/tip/pc/services/hw/task/taskflow/usecases/userActivityTMScope.xsl"/>
<xsl:include href="/oracle/tip/pc/services/hw/task/taskflow/usecases/userActivityInitiateTask.xsl"/>
<xsl:include href="/oracle/tip/pc/services/hw/task/taskflow/usecases/userActivityCommonAssignment.xsl"/>

<!-- Client passed params -->
<xsl:param name="GENERATE_IN_DEBUG_MODE_PARAM" select="'false'"/>
<xsl:param name="partnerLinkParam" select="''"/>
<xsl:param name="routingServicePartnerLinkParam" select="''"/>
<xsl:param name="scopeNameParam" select="''"/>
<xsl:param name="taskVariableNameParam" select="''"/>
<xsl:param name="maxUserParam" select="'false'"/>
<xsl:param name="maxLevelParam" select="'false'"/>
<xsl:param name="maxTitleParam" select="'false'"/>
<xsl:param name="continueTaskParam" select="'false'"/>
<xsl:param name="taskActionHandlerPartnerLinkParam" select="''"/>

<!-- Generator passed params -->
<xsl:param name="taskPrefixParam" select="''"/>
<xsl:param name="taskWSDLPrefixParam" select="''"/>
<xsl:param name="routingServiceWSDLPrefixParam" select="''"/>
<xsl:param name="taskManagerMessageParam" select="''"/>
<xsl:param name="taskUpdateMessageParam" select="''"/>
<xsl:param name="taskIdCorrelationSetParam" select="''"/>
<xsl:param name="expDurationVariableParam" select="''"/>
<xsl:param name="taskErroredMessageParam" select="''"/>
<xsl:param name="identityServiceWSDLPrefixParam" select="''"/>
<xsl:param name="tempVariableParam" select="'false'"/>
<xsl:param name="taskActionHandlerWSDLPrefixParam" select="''"/>
<xsl:param name="managementChainElementParam" select="''"/>
<xsl:param name="escalateTaskOnExpirationVariableNameParam" select="''"/>

<xsl:template match="/">

<scope xmlns="http://schemas.xmlsoap.org/ws/2003/03/business-process/"
   xmlns:xsd="http://www.w3.org/2001/XMLSchema"
   xmlns:bpws="http://schemas.xmlsoap.org/ws/2003/03/business-process/" 
   xmlns:bpelx="http://schemas.oracle.com/bpel/extension"
   xmlns:task="http://xmlns.oracle.com/pcbpel/taskservice/task"
   xmlns:taskmngr="http://xmlns.oracle.com/pcbpel/taskservice/taskmanager"
   xmlns:taskroutingservice="http://xmlns.oracle.com/pcbpel/taskservice/taskroutingservice"
   xmlns:taskactionhandler="http://xmlns.oracle.com/pcbpel/taskservice/taskActionHandler"
   xmlns:identityservice="http://xmlns.oracle.com/pcbpel/identityservice/local"
   xmlns:ora="http://schemas.oracle.com/xpath/extension"
   name="{$scopeNameParam}" variableAccessSerializable="no">
    
   <variables>
      <variable name="{$taskManagerMessageParam}" 
                messageType="{$taskWSDLPrefixParam}:taskMessage"/>
      <variable name="{$escalateTaskOnExpirationVariableNameParam}" 
                messageType="{$routingServiceWSDLPrefixParam}:taskEscalateOnExpirationMessage"/>
      <variable name="{$taskErroredMessageParam}" 
                messageType="{$taskWSDLPrefixParam}:taskErroredMessage"/>
      <variable name="{$expDurationVariableParam}" type="xsd:duration"/>
      <variable name="{$tempVariableParam}" type="xsd:string"/>
      <variable name="{$managementChainElementParam}" 
                element="{$identityServiceWSDLPrefixParam}:managementChain"/>
   </variables>
       
   <correlationSets>
      <correlationSet name="{$taskIdCorrelationSetParam}"
                        properties="{$taskWSDLPrefixParam}:taskId"/>
   </correlationSets>

   <sequence>

      <xsl:copy-of select="/input/bpel:assign"/>

      <!-- Set the number of times renewed and expiration duration -->
      <assign name="setEscalationPolicy">
         <copy>
            <from variable="{$taskVariableNameParam}" query="/{$taskPrefixParam}:task/{$taskPrefixParam}:expirationDuration"/>
            <to variable="{$expDurationVariableParam}"/>
         </copy>
         <copy>
            <from expression="string('{$maxTitleParam}')"/>
            <to variable="{$managementChainElementParam}" query="/{$identityServiceWSDLPrefixParam}:managementChain/{$identityServiceWSDLPrefixParam}:uptoTitle"/>
         </copy>
         <copy>
            <from expression="string('{$maxLevelParam}')"/>
            <to variable="{$managementChainElementParam}" query="/{$identityServiceWSDLPrefixParam}:managementChain/{$identityServiceWSDLPrefixParam}:levels"/>
         </copy>
      </assign>

      <!-- Set the system attributes -->
      <assign name="setSystemDefinedAttributes">
         <xsl:call-template name="commonAssignment">
            <xsl:with-param name="taskPrefixParam" select="$taskPrefixParam"/>
            <xsl:with-param name="taskVariableNameParam" select="$taskVariableNameParam"/>
            <xsl:with-param name="continueTaskParam" select="$continueTaskParam"/>
         </xsl:call-template>
         <copy>
             <from expression="string('SINGLE_APPROVAL_WITH_ESCALATION')"/>
             <to variable="{$taskVariableNameParam}" query="/{$taskPrefixParam}:task/{$taskPrefixParam}:pattern"/>
         </copy>
         <copy>
             <from expression="false()"/>
             <to variable="{$taskVariableNameParam}" query="/{$taskPrefixParam}:task/{$taskPrefixParam}:hasSubTasks"/>
         </copy>
         <copy>
            <from variable="{$taskVariableNameParam}"/>
            <to variable="{$taskManagerMessageParam}" part="payload"/>
         </copy>
      </assign>

      <xsl:call-template name="initiateTaskScope">
         <xsl:with-param name="partnerLinkParam" select="$partnerLinkParam"/>
         <xsl:with-param name="taskWSDLPrefixParam" select="$taskWSDLPrefixParam"/>
         <xsl:with-param name="taskManagerMessageParam" select="$taskManagerMessageParam"/>
         <xsl:with-param name="taskIdCorrelationSetParam" select="$taskIdCorrelationSetParam"/>
         <xsl:with-param name="taskErroredMessageParam" select="$taskErroredMessageParam"/>
         <xsl:with-param name="continueTaskParam" select="$continueTaskParam"/>
      </xsl:call-template>

      <!-- Iterate until the task is marked as EXPIRED, COMPLETED, WITHDRAWN or ERRORED -->
      <while condition="bpws:getVariableData('{$taskManagerMessageParam}', 'payload', '/{$taskPrefixParam}:task/{$taskPrefixParam}:state') != 'EXPIRED' and bpws:getVariableData('{$taskManagerMessageParam}', 'payload', '/{$taskPrefixParam}:task/{$taskPrefixParam}:state') != 'COMPLETED' and bpws:getVariableData('{$taskManagerMessageParam}', 'payload', '/{$taskPrefixParam}:task/{$taskPrefixParam}:state') != 'WITHDRAWN' and bpws:getVariableData('{$taskManagerMessageParam}', 'payload', '/{$taskPrefixParam}:task/{$taskPrefixParam}:state') != 'ERRORED'">

          <sequence>

             <xsl:call-template name="mainScope">
                <xsl:with-param name="partnerLinkParam" select="$partnerLinkParam"/>
                <xsl:with-param name="taskActionHandlerPartnerLinkParam"
                                select="$taskActionHandlerPartnerLinkParam"/>
                <xsl:with-param name="taskPrefixParam" select="$taskPrefixParam"/>
                <xsl:with-param name="taskActionHandlerWSDLPrefixParam"
                                select="$taskActionHandlerWSDLPrefixParam"/>
                <xsl:with-param name="taskWSDLPrefixParam" select="$taskWSDLPrefixParam"/>
                <xsl:with-param name="taskManagerMessageParam" select="$taskManagerMessageParam"/>
                <xsl:with-param name="taskUpdateMessageParam" select="$taskUpdateMessageParam"/>
                <xsl:with-param name="taskIdCorrelationSetParam" select="$taskIdCorrelationSetParam"/>
                <xsl:with-param name="taskErroredMessageParam" select="$taskErroredMessageParam"/>
             </xsl:call-template>


             <switch>
                <case condition="bpws:getVariableData('{$taskManagerMessageParam}',
                                                      'payload',
                                                      '/{$taskPrefixParam}:task/{$taskPrefixParam}:state') = 'EXPIRED'">
                   <bpelx:annotation>
                      <bpelx:pattern><xsl:text>Task is EXPIRED</xsl:text></bpelx:pattern>
                   </bpelx:annotation>

                   <scope name="escalateTask">
                      <faultHandlers>
                         <catch faultName="{$routingServiceWSDLPrefixParam}:taskErroredFault"
                                faultVariable="{$taskErroredMessageParam}">
                            <sequence>
                               <assign name="readErroredTask">
                                  <copy>
                                     <from variable="{$taskErroredMessageParam}" 
                                           part="payload"/>
                                     <to variable="{$taskManagerMessageParam}" 
                                         part="payload"/>
                                   </copy>
                               </assign>
                            </sequence>
                         </catch>
                      </faultHandlers>

                      <sequence>

                         <assign name="setEscalationMessage">
                            <copy>
                               <from variable="{$taskManagerMessageParam}" part="payload"/>
                               <to variable="{$escalateTaskOnExpirationVariableNameParam}" part="task"/>
                            </copy>
                            <copy>
                               <from variable="{$managementChainElementParam}"/>
                               <to variable="{$escalateTaskOnExpirationVariableNameParam}" part="managementChain"/>
                            </copy>
                            <copy>
                               <from variable="{$expDurationVariableParam}"/>
                               <to variable="{$escalateTaskOnExpirationVariableNameParam}" part="renewDuration"/>
                            </copy>
                         </assign>
   
                         <!--  Escalate task -->
                         <invoke name="escalateTask" 
                                 partnerLink="{$routingServicePartnerLinkParam}" 
                                 portType="{$routingServiceWSDLPrefixParam}:TaskRoutingService" 
                                 operation="escalateTaskOnExpiration" 
                                 inputVariable="{$escalateTaskOnExpirationVariableNameParam}"
                                 outputVariable="{$taskManagerMessageParam}"/>

                         <assign name="updateEscalationPolicy">
                            <copy>
                               <from expression="number(bpws:getVariableData('{$managementChainElementParam}', '/{$identityServiceWSDLPrefixParam}:managementChain/{$identityServiceWSDLPrefixParam}:levels')) - 1"/>
                               <to variable="{$managementChainElementParam}" query="/{$identityServiceWSDLPrefixParam}:managementChain/{$identityServiceWSDLPrefixParam}:levels"/>
                            </copy>
                         </assign>
                      </sequence>
                   </scope>
                </case>
             </switch>
          </sequence>
      </while>

      <!-- Read task document from taskMessage -->
      <assign name="readUpdatedTask">
         <copy>
            <from variable="{$taskManagerMessageParam}" part="payload"/>
            <to variable="{$taskVariableNameParam}"/>
         </copy>
      </assign>

   </sequence>

</scope>

</xsl:template>

</xsl:stylesheet>
