<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:tt="http://xmlns.oracle.com/pcbpel/taskservice/tasktype"
xmlns:bpel="http://schemas.xmlsoap.org/ws/2003/03/business-process/">

<xsl:include href="/oracle/tip/pc/services/hw/task/taskflow/usecases/userActivityTMScope.xsl"/>
<xsl:include href="/oracle/tip/pc/services/hw/task/taskflow/usecases/userActivityInitiateTask.xsl"/>
<xsl:include href="/oracle/tip/pc/services/hw/task/taskflow/usecases/userActivityCommonAssignment.xsl"/>

<!-- Client passed params -->
<xsl:param name="GENERATE_IN_DEBUG_MODE_PARAM" select="'false'"/>
<xsl:param name="partnerLinkParam" select="''"/>
<xsl:param name="scopeNameParam" select="''"/>
<xsl:param name="taskVariableNameParam" select="''"/>
<xsl:param name="taskManagerRenewMessageParam" select="''"/>
<xsl:param name="numberRenewedParam" select="''"/>
<xsl:param name="continueTaskParam" select="'false'"/>
<xsl:param name="taskActionHandlerPartnerLinkParam" select="''"/>

<!-- Generator passed params -->
<xsl:param name="taskPrefixParam" select="''"/>
<xsl:param name="taskWSDLPrefixParam" select="''"/>
<xsl:param name="taskManagerMessageParam" select="''"/>
<xsl:param name="taskUpdateMessageParam" select="''"/>
<xsl:param name="taskIdCorrelationSetParam" select="''"/>
<xsl:param name="numOfTimesRenewedVariableParam" select="''"/>
<xsl:param name="renewDurationVariableParam" select="''"/>
<xsl:param name="maxRenewVariableParam" select="''"/>
<xsl:param name="taskErroredMessageParam" select="''"/>
<xsl:param name="tempVariableParam" select="'false'"/>
<xsl:param name="taskActionHandlerWSDLPrefixParam" select="''"/>

<xsl:template match="/">

<scope xmlns="http://schemas.xmlsoap.org/ws/2003/03/business-process/"
   xmlns:xsd="http://www.w3.org/2001/XMLSchema"
   xmlns:bpws="http://schemas.xmlsoap.org/ws/2003/03/business-process/" 
   xmlns:bpelx="http://schemas.oracle.com/bpel/extension"
   xmlns:task="http://xmlns.oracle.com/pcbpel/taskservice/task"
   xmlns:taskmngr="http://xmlns.oracle.com/pcbpel/taskservice/taskmanager"
   xmlns:taskactionhandler="http://xmlns.oracle.com/pcbpel/taskservice/taskActionHandler"
   xmlns:ora="http://schemas.oracle.com/xpath/extension"
   name="{$scopeNameParam}" variableAccessSerializable="no">
    
   <variables>
      <variable name="{$taskManagerMessageParam}" 
                messageType="{$taskWSDLPrefixParam}:taskMessage"/>
      <variable name="{$taskManagerRenewMessageParam}" 
                messageType="{$taskWSDLPrefixParam}:taskRenewMessage"/>
      <variable name="{$taskErroredMessageParam}" 
                messageType="{$taskWSDLPrefixParam}:taskErroredMessage"/>
      <variable name="{$numOfTimesRenewedVariableParam}" type="xsd:int"/>
      <variable name="{$maxRenewVariableParam}" type="xsd:int"/>
      <variable name="{$renewDurationVariableParam}" type="xsd:duration"/>
      <variable name="{$tempVariableParam}" type="xsd:string"/>
   </variables>
       
   <correlationSets>
      <correlationSet name="{$taskIdCorrelationSetParam}"
                        properties="{$taskWSDLPrefixParam}:taskId"/>
   </correlationSets>

   <sequence>

      <xsl:copy-of select="/input/bpel:assign"/>

      <!-- Set the number of times renewed and renew duration -->
      <assign name="setRenewalPolicy">
         <copy>
            <from variable="{$taskVariableNameParam}" query="/{$taskPrefixParam}:task/{$taskPrefixParam}:expirationDuration"/>
            <to variable="{$renewDurationVariableParam}"/>
         </copy>
         <copy>
            <from expression="number({$numberRenewedParam})"/>
            <to variable="{$maxRenewVariableParam}"/>
         </copy>
      </assign>

      <!-- Set the system attributes -->
      <assign name="setSystemDefinedAttributes">
         <xsl:call-template name="commonAssignment">
            <xsl:with-param name="taskPrefixParam" select="$taskPrefixParam"/>
            <xsl:with-param name="taskVariableNameParam" select="$taskVariableNameParam"/>
            <xsl:with-param name="continueTaskParam" select="$continueTaskParam"/>
         </xsl:call-template>
         <copy>
             <from expression="string('SINGLE_APPROVAL_WITH_RENEWAL')"/>
             <to variable="{$taskVariableNameParam}" query="/{$taskPrefixParam}:task/{$taskPrefixParam}:pattern"/>
         </copy>
         <copy>
             <from expression="false()"/>
             <to variable="{$taskVariableNameParam}" query="/{$taskPrefixParam}:task/{$taskPrefixParam}:hasSubTasks"/>
         </copy>
         <copy>
            <from expression="number(0)"/>
            <to variable="{$numOfTimesRenewedVariableParam}"/>
         </copy>
         <!-- Assign task document to the message sent to the task manager -->
         <copy>
            <from variable="{$taskVariableNameParam}"/>
            <to variable="{$taskManagerMessageParam}" part="payload"/>
         </copy>
      </assign>

      <xsl:call-template name="initiateTaskScope">
         <xsl:with-param name="partnerLinkParam" select="$partnerLinkParam"/>
         <xsl:with-param name="taskWSDLPrefixParam" select="$taskWSDLPrefixParam"/>
         <xsl:with-param name="taskManagerMessageParam" select="$taskManagerMessageParam"/>
         <xsl:with-param name="taskIdCorrelationSetParam" select="$taskIdCorrelationSetParam"/>
         <xsl:with-param name="taskErroredMessageParam" select="$taskErroredMessageParam"/>
         <xsl:with-param name="continueTaskParam" select="$continueTaskParam"/>
      </xsl:call-template>

      <!-- Iterate until the task is marked as EXPIRED, COMPLETED, WITHDRAWN or ERRORED -->
      <while condition="bpws:getVariableData('{$taskManagerMessageParam}', 'payload', '/{$taskPrefixParam}:task/{$taskPrefixParam}:state') != 'EXPIRED' and bpws:getVariableData('{$taskManagerMessageParam}', 'payload', '/{$taskPrefixParam}:task/{$taskPrefixParam}:state') != 'COMPLETED' and bpws:getVariableData('{$taskManagerMessageParam}', 'payload', '/{$taskPrefixParam}:task/{$taskPrefixParam}:state') != 'WITHDRAWN' and bpws:getVariableData('{$taskManagerMessageParam}', 'payload', '/{$taskPrefixParam}:task/{$taskPrefixParam}:state') != 'ERRORED'">

          <sequence>

             <xsl:call-template name="mainScope">
                <xsl:with-param name="partnerLinkParam" select="$partnerLinkParam"/>
                <xsl:with-param name="taskActionHandlerPartnerLinkParam"
                                select="$taskActionHandlerPartnerLinkParam"/>
                <xsl:with-param name="taskPrefixParam" select="$taskPrefixParam"/>
                <xsl:with-param name="taskWSDLPrefixParam" select="$taskWSDLPrefixParam"/>
                <xsl:with-param name="taskActionHandlerWSDLPrefixParam"
                                select="$taskActionHandlerWSDLPrefixParam"/>
                <xsl:with-param name="taskManagerMessageParam" select="$taskManagerMessageParam"/>
                <xsl:with-param name="taskUpdateMessageParam" select="$taskUpdateMessageParam"/>
                <xsl:with-param name="taskIdCorrelationSetParam" select="$taskIdCorrelationSetParam"/>
                <xsl:with-param name="taskErroredMessageParam" select="$taskErroredMessageParam"/>
             </xsl:call-template>

             <switch name="checkRenewalPolicy">
                <case condition="bpws:getVariableData('{$taskManagerMessageParam}',
                                                      'payload',
                                                      '/{$taskPrefixParam}:task/{$taskPrefixParam}:state') = 'EXPIRED' and bpws:getVariableData('{$maxRenewVariableParam}') >= (bpws:getVariableData('{$numOfTimesRenewedVariableParam}') + 1)">

                   <bpelx:annotation>
                      <bpelx:pattern><xsl:text>Task is EXPIRED and the renewal criteria is satisfied</xsl:text></bpelx:pattern>
                   </bpelx:annotation>

                   <scope name="renewTask">
                      <faultHandlers>
                         <catch faultName="{$taskWSDLPrefixParam}:taskErroredFault"
                                faultVariable="{$taskErroredMessageParam}">
                            <sequence>
                               <assign name="readErroredTask">
                                  <copy>
                                     <from variable="{$taskErroredMessageParam}" 
                                           part="payload"/>
                                     <to variable="{$taskManagerMessageParam}" 
                                         part="payload"/>
                                   </copy>
                               </assign>
                            </sequence>
                         </catch>
                      </faultHandlers>
                               
                      <sequence>

                         <assign name="setRenewMessage">
                            <copy>
                               <from expression="bpws:getVariableData('{$numOfTimesRenewedVariableParam}') + 1"/>
                               <to variable="{$numOfTimesRenewedVariableParam}"/>
                            </copy>
                            <copy>
                               <from variable="{$renewDurationVariableParam}"/>
                               <to variable="{$taskManagerRenewMessageParam}"
                                   part="duration"/>
                            </copy>
                            <copy>
                               <from variable="{$taskManagerMessageParam}"
                                     part="payload"/>
                               <to variable="{$taskManagerRenewMessageParam}"
                                   part="payload"/>
                            </copy>
                         </assign>

                         <!--  Renew task -->
                         <invoke name="renewTask" 
                                 partnerLink="{$partnerLinkParam}" 
                                 portType="{$taskWSDLPrefixParam}:TaskManager" 
                                 operation="renewTask" 
                                 inputVariable="{$taskManagerRenewMessageParam}"
                                 outputVariable="{$taskManagerMessageParam}"/>
                      </sequence>
                   </scope>
                </case>
             </switch>
          </sequence>

      </while>

      <!-- Read task document from taskMessage -->
      <assign name="readUpdatedTask">
         <copy>
            <from variable="{$taskManagerMessageParam}" part="payload"/>
            <to variable="{$taskVariableNameParam}"/>
         </copy>
      </assign>

   </sequence>

</scope>

</xsl:template>

</xsl:stylesheet>
