<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0"
xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:bpel="http://schemas.xmlsoap.org/ws/2003/03/business-process/">

<xsl:include href="/oracle/tip/pc/services/hw/task/taskflow/usecases/userActivityTMScope.xsl"/>
<xsl:include href="/oracle/tip/pc/services/hw/task/taskflow/usecases/userActivityInitiateTask.xsl"/>
<xsl:include href="/oracle/tip/pc/services/hw/task/taskflow/usecases/userActivityCommonAssignment.xsl"/>

<!-- Client passed params -->
<xsl:param name="GENERATE_IN_DEBUG_MODE_PARAM" select="'false'"/>
<xsl:param name="partnerLinkParam" select="''"/>
<xsl:param name="scopeNameParam" select="''"/>
<xsl:param name="taskVariableNameParam" select="''"/>
<xsl:param name="areAssigneesGroupsParam" select="''"/>
<xsl:param name="continueTaskParam" select="'false'"/>
<xsl:param name="taskActionHandlerPartnerLinkParam" select="''"/>
<xsl:param name="conclusionPercentageParam" select="''"/>
<xsl:param name="defaultConclusionParam" select="''"/>
<xsl:param name="earlyCompletionParam" select="'false'"/>

<!-- Generator passed params -->
<xsl:param name="taskPrefixParam" select="''"/>
<xsl:param name="taskWSDLPrefixParam" select="''"/>
<xsl:param name="taskManagerMessageParam" select="''"/>
<xsl:param name="subTaskManagerMessageParam" select="''"/>
<xsl:param name="taskUpdateMessageParam" select="''"/>
<xsl:param name="taskIdCorrelationSetParam" select="''"/>
<xsl:param name="taskErroredMessageParam" select="''"/>
<xsl:param name="assigneeEntitiesParam" select="''"/>
<xsl:param name="conclusionVariablesPrefixParam" select="''"/>
<xsl:param name="numberOfSubTasksVariableParam" select="''"/>
<xsl:param name="conclusionPercentageVariableParam" select="''"/>
<xsl:param name="defaultConclusionVariableParam" select="''"/>
<xsl:param name="completeTaskInputMessage" select="''"/>
<xsl:param name="tempVariableParam" select="'false'"/>
<xsl:param name="taskActionHandlerWSDLPrefixParam" select="''"/>
<xsl:param name="subTaskAssignmentElementParam" select="''"/>
<xsl:param name="initSubTaskMessageParam" select="''"/>

<xsl:template name="parallelCore" match="/">

      <assign name="setOutcomeDeterminationPolicy">
         <copy>
             <from expression="number({$conclusionPercentageParam})"/>
             <to variable="{$conclusionPercentageVariableParam}"/>
         </copy>
         <copy>
             <from expression="string('{$defaultConclusionParam}')"/>
             <to variable="{$defaultConclusionVariableParam}"/>
         </copy>
      </assign>

      <assign name="setSystemDefinedAttributes">
         <xsl:call-template name="commonAssignment">
            <xsl:with-param name="taskPrefixParam" select="$taskPrefixParam"/>
            <xsl:with-param name="taskVariableNameParam" select="$taskVariableNameParam"/>
            <xsl:with-param name="continueTaskParam" select="$continueTaskParam"/>
         </xsl:call-template>
         <copy>
             <from expression="string('PARALLEL')"/>
             <to variable="{$taskVariableNameParam}" query="/{$taskPrefixParam}:task/{$taskPrefixParam}:pattern"/>
         </copy>
         <copy>
             <from expression="true()"/>
             <to variable="{$taskVariableNameParam}" query="/{$taskPrefixParam}:task/{$taskPrefixParam}:hasSubTasks"/>
         </copy>
         <copy>
            <from expression="ora:countNodes('{$assigneeEntitiesParam}', '/taskmngr:assigneeEntities/taskmngr:assignee')"/>
            <to variable="{$numberOfSubTasksVariableParam}"/>
         </copy>
         <!--copy>
            <from expression="number(0)"/>
            <to variable="{$conclusionVariablesPrefixParam}Errored"/>
         </copy>
         <copy>
            <from expression="number(0)"/>
            <to variable="{$conclusionVariablesPrefixParam}Withdrawn"/>
         </copy>
         <copy>
            <from expression="number(0)"/>
            <to variable="{$conclusionVariablesPrefixParam}Expired"/>
         </copy-->
         <xsl:for-each select="/input/conclusions/conclusion">
         <copy>
            <from expression="number(0)"/>
            <to variable="{$conclusionVariablesPrefixParam}{@name}"/>
         </copy>
         </xsl:for-each>
         <!-- Assign task document to the message sent to the task manager -->
         <copy>
            <from variable="{$taskVariableNameParam}"/>
            <to variable="{$taskManagerMessageParam}" part="payload"/>
         </copy>
      </assign>

      <scope name="initiateMainTask">
         <faultHandlers>
            <catch faultName="{$taskWSDLPrefixParam}:taskErroredFault"
                   faultVariable="{$taskErroredMessageParam}">
               <assign name="readErroredTask">
                  <copy>
                     <from variable="{$taskErroredMessageParam}" part="payload"/>
                     <to variable="{$taskManagerMessageParam}" part="payload"/>
                  </copy>
               </assign>
            </catch>
         </faultHandlers>

         <sequence>
            <xsl:choose>
            <xsl:when test="$continueTaskParam = 'true'">
            <!--  reinitiate task -->
            <invoke name="reinitiateMainTask" 
                    partnerLink="{$partnerLinkParam}" 
                    portType="{$taskWSDLPrefixParam}:TaskManager" 
                    operation="reinitiateTask" 
                    inputVariable="{$taskManagerMessageParam}"
                    outputVariable="{$taskManagerMessageParam}"/>
            </xsl:when>
            <xsl:otherwise>
            <!--  initiate task -->
            <invoke name="initiateMainTask" 
                    partnerLink="{$partnerLinkParam}" 
                    portType="{$taskWSDLPrefixParam}:TaskManager" 
                    operation="initiateTask" 
                    inputVariable="{$taskManagerMessageParam}"
                    outputVariable="{$taskManagerMessageParam}"/>
            </xsl:otherwise>
            </xsl:choose>
            <assign name="readInitiatedTask">
               <!-- Assign task document from the reply from the task manager -->
               <copy>
                  <from variable="{$taskManagerMessageParam}" part="payload"/>
                  <to variable="{$taskVariableNameParam}"/>
               </copy>
            </assign>
         </sequence>
      </scope>

      <scope name="executeSubTasks">
      <xsl:choose>
      <xsl:when test="$earlyCompletionParam = 'true'">
         <faultHandlers>
            <catch faultName="{$taskWSDLPrefixParam}:ParallelTaskEarlyCompletion">
               <empty/>
            </catch>
         </faultHandlers>
      </xsl:when>
      </xsl:choose>

      <!-- create flown -->
      <bpelx:flowN N="number(bpws:getVariableData('{$numberOfSubTasksVariableParam}'))" 
                   indexVariable="index"    
                   xmlns:bpelx="http://schemas.oracle.com/bpel/extension">
         <scope name="executeSubTask">

            <variables>
               <variable name="{$initSubTaskMessageParam}" 
                         messageType="{$taskWSDLPrefixParam}:initiateSubTaskMessage"/>
               <variable name="{$subTaskAssignmentElementParam}" 
                         element="{$taskWSDLPrefixParam}:taskAssignment"/>
               <variable name="{$subTaskManagerMessageParam}" 
                         messageType="{$taskWSDLPrefixParam}:taskMessage"/>
            </variables>

            <correlationSets>
               <correlationSet name="{$taskIdCorrelationSetParam}"
                                        properties="{$taskWSDLPrefixParam}:taskId"/>
            </correlationSets>

            <sequence>

               <assign name="setInitiateSubTaskMessage">
                  <xsl:choose>
                  <xsl:when test="$areAssigneesGroupsParam = 'true()'">
                  <copy>
                     <from variable="{$assigneeEntitiesParam}" 
                           query="/taskmngr:assigneeEntities/taskmngr:assignee[bpws:getVariableData('index')]"/>
                     <to variable="{$subTaskAssignmentElementParam}" 
                         query="/{$taskWSDLPrefixParam}:taskAssignment/{$taskWSDLPrefixParam}:assigneeGroups[1]"/>
                  </copy>
                  </xsl:when>
                  <xsl:otherwise>
                  <copy>
                     <from variable="{$assigneeEntitiesParam}" 
                           query="/taskmngr:assigneeEntities/taskmngr:assignee[bpws:getVariableData('index')]"/>
                     <to variable="{$subTaskAssignmentElementParam}" 
                         query="/{$taskWSDLPrefixParam}:taskAssignment/{$taskWSDLPrefixParam}:assigneeUsers[1]"/>
                  </copy>
                  </xsl:otherwise>
                  </xsl:choose>
                  <copy>
                     <from variable="{$taskVariableNameParam}"/>
                     <to variable="{$initSubTaskMessageParam}" part="payload"/>
                  </copy>
                  <copy>
                     <from variable="{$subTaskAssignmentElementParam}"/>
                     <to variable="{$initSubTaskMessageParam}" part="taskAssignment"/>
                  </copy>
               </assign>

               <xsl:call-template name="initiateTaskScope">
                  <xsl:with-param name="partnerLinkParam" select="$partnerLinkParam"/>
                  <xsl:with-param name="taskWSDLPrefixParam" select="$taskWSDLPrefixParam"/>
                  <xsl:with-param name="taskManagerMessageParam" select="$initSubTaskMessageParam"/>
                  <xsl:with-param name="taskIdCorrelationSetParam" select="$taskIdCorrelationSetParam"/>
                  <xsl:with-param name="taskErroredMessageParam" select="$taskErroredMessageParam"/>
                  <xsl:with-param name="continueTaskParam" select="'false'"/>
                  <xsl:with-param name="initSubTaskParam" select="'true'"/>
                  <xsl:with-param name="outputTaskManagerMessageParam" select="$subTaskManagerMessageParam"/>
               </xsl:call-template>

               <xsl:call-template name="mainScope">
                  <xsl:with-param name="partnerLinkParam" select="$partnerLinkParam"/>
                  <xsl:with-param name="taskActionHandlerPartnerLinkParam"
                                                select="$taskActionHandlerPartnerLinkParam"/>
                  <xsl:with-param name="taskPrefixParam" select="$taskPrefixParam"/>
                  <xsl:with-param name="taskWSDLPrefixParam" select="$taskWSDLPrefixParam"/>
                  <xsl:with-param name="taskActionHandlerWSDLPrefixParam"
                                                 select="$taskActionHandlerWSDLPrefixParam"/>
                  <xsl:with-param name="taskManagerMessageParam" select="$subTaskManagerMessageParam"/>
                  <xsl:with-param name="taskUpdateMessageParam" select="$taskUpdateMessageParam"/>
                  <xsl:with-param name="taskIdCorrelationSetParam" select="$taskIdCorrelationSetParam"/>
                  <xsl:with-param name="taskErroredMessageParam" select="$taskErroredMessageParam"/>
               </xsl:call-template>

               <switch name="setOutcomeCount">
                  <xsl:for-each select="/input/conclusions/conclusion">
                  <case condition="bpws:getVariableData('{$subTaskManagerMessageParam}', 'payload', '/{$taskPrefixParam}:task/{$taskPrefixParam}:state') = 'COMPLETED' and bpws:getVariableData('{$subTaskManagerMessageParam}', 'payload', '/{$taskPrefixParam}:task/{$taskPrefixParam}:conclusion') = '{@name}'">
                     <bpelx:annotation>
                        <bpelx:pattern><xsl:text>Task is COMPLETED and outcome is </xsl:text><xsl:value-of select="@name"/></bpelx:pattern>
                     </bpelx:annotation>
                     <sequence>
                        <assign name="setNumberOfOutcome{@name}">
                           <copy>
                              <from expression="1 + bpws:getVariableData('{$conclusionVariablesPrefixParam}{@name}')"/>
                              <to variable="{$conclusionVariablesPrefixParam}{@name}"/>
                           </copy>
                        </assign>

                        <xsl:choose>
                        <xsl:when test="$earlyCompletionParam = 'true'">
                        <switch name="setTaskConclusion" xmlns:bpelx="http://schemas.oracle.com/bpel/extension">
                           <case condition="number(100 * (bpws:getVariableData('{$conclusionVariablesPrefixParam}{@name}') div bpws:getVariableData('{$numberOfSubTasksVariableParam}'))) >= bpws:getVariableData('{$conclusionPercentageVariableParam}')">
                              <bpelx:annotation>
                                 <bpelx:pattern><xsl:text>Early completion of flown is reached</xsl:text></bpelx:pattern>
                              </bpelx:annotation>

                              <throw faultName="{$taskWSDLPrefixParam}:ParallelTaskEarlyCompletion"/>
                           </case>
                        </switch>
                        </xsl:when>
                        </xsl:choose>

                     </sequence>
                  </case>
                  </xsl:for-each>

                  <!--case condition="bpws:getVariableData('{$subTaskManagerMessageParam}', 'payload', '/{$taskPrefixParam}:task/{$taskPrefixParam}:state') = 'EXPIRED'">
                     <sequence>
                        <assign name="setExpiredResults">
                           <copy>
                              <from expression="1 + bpws:getVariableData('{$conclusionVariablesPrefixParam}Expired')"/>
                              <to variable="{$conclusionVariablesPrefixParam}Expired"/>
                           </copy>
                        </assign>
                     </sequence>
                  </case>

                  <case condition="bpws:getVariableData('{$subTaskManagerMessageParam}', 'payload', '/{$taskPrefixParam}:task/{$taskPrefixParam}:state') = 'WITHDRAWN'">
                     <sequence>
                        <assign name="setWithdrawnResults">
                           <copy>
                              <from expression="1 + bpws:getVariableData('{$conclusionVariablesPrefixParam}Withdrawn')"/>
                              <to variable="{$conclusionVariablesPrefixParam}Withdrawn"/>
                           </copy>
                        </assign>
                     </sequence>
                  </case>

                  <case condition="bpws:getVariableData('{$subTaskManagerMessageParam}', 'payload', '/{$taskPrefixParam}:task/{$taskPrefixParam}:state') = 'ERRORED'">
                     <sequence>
                        <assign name="setErroredResults">
                           <copy>
                              <from expression="1 + bpws:getVariableData('{$conclusionVariablesPrefixParam}Errored')"/>
                              <to variable="{$conclusionVariablesPrefixParam}Errored"/>
                           </copy>
                        </assign>
                     </sequence>
                  </case-->
               </switch>

            </sequence>

         </scope>
      </bpelx:flowN>

      </scope>

      <scope name="completeMainTask">
         <faultHandlers>
            <catch faultName="{$taskWSDLPrefixParam}:taskErroredFault"
                   faultVariable="{$taskErroredMessageParam}">
               <assign name="readErroredTask">
                  <copy>
                     <from variable="{$taskErroredMessageParam}" part="payload"/>
                     <to variable="{$taskManagerMessageParam}" part="payload"/>
                  </copy>
               </assign>
            </catch>
         </faultHandlers>

         <sequence>

            <switch name="setTaskConclusion" xmlns:bpelx="http://schemas.oracle.com/bpel/extension">
               <xsl:for-each select="/input/conclusions/conclusion">
               <case condition="number(100 * (bpws:getVariableData('{$conclusionVariablesPrefixParam}{@name}') div bpws:getVariableData('{$numberOfSubTasksVariableParam}'))) >= bpws:getVariableData('{$conclusionPercentageVariableParam}')">
                  <bpelx:annotation>
                     <bpelx:pattern><xsl:text>Number of tasks with outcome </xsl:text><xsl:value-of select="@name"/><xsl:text> is greater than the percentage of total tasks specified in the outcome determination policy</xsl:text></bpelx:pattern>
                  </bpelx:annotation>

                  <sequence>
                     <assign name="setOutcomeTo{@name}">
                       <copy>
                          <from expression="string('{@name}')"/>
                          <to variable="{$completeTaskInputMessage}" part="conclusion"/>
                       </copy>
                     </assign>
                  </sequence>
               </case>
               </xsl:for-each>
               <otherwise>
                  <bpelx:annotation>
                     <bpelx:pattern><xsl:text>Set default conclusion</xsl:text></bpelx:pattern>
                  </bpelx:annotation>
                  <sequence>
                     <assign name="setDefaultOutcome">
                       <copy>
                          <from expression="bpws:getVariableData('{$defaultConclusionVariableParam}')"/>
                          <to variable="{$completeTaskInputMessage}" part="conclusion"/>
                       </copy>
                     </assign>
                  </sequence>
               </otherwise>
            </switch>

            <!-- Assign task document to the message sent to the task manager -->
            <assign name="createTaskCompleteMessage">
               <copy>
                  <from variable="{$taskVariableNameParam}" query="/{$taskPrefixParam}:task/{$taskPrefixParam}:taskId"/>
                  <to variable="{$completeTaskInputMessage}" part="taskId"/>
               </copy>
            </assign>

            <!--  complete task -->
            <invoke name="completeTask" 
                    partnerLink="{$partnerLinkParam}" 
                    portType="{$taskWSDLPrefixParam}:TaskManager" 
                    operation="completeTask" 
                    inputVariable="{$completeTaskInputMessage}"
                    outputVariable="{$taskManagerMessageParam}"/>

         </sequence>
      </scope>


</xsl:template>

</xsl:stylesheet>
